﻿// This code is public domain.

package 
{
	import flash.display.MovieClip;
    import flash.events.*;
	import flash.geom.Point;
	import fl.motion.Color;
	import flash.text.*;
	import flash.media.*;
	import flash.net.*;
	
	public class Main extends MovieClip 
	{
		private var BGM:Sound;
		private var BGMChannel:SoundChannel;
		
		private var m_StatusText:TextField;
		
		//public static var instance:MovieClip;
		
		private var m_Grid:Array;				// This stores a 2d array reperesentaion of the game grid.
		private var m_SnakeList:Array;			// This stores a list of all the snake parts.
		private var m_Fruit:Fruit;			    // The fruit that moves and grows the snake.
		
		private var m_MoveCountdown:int;		// How many frames before the head moves.
		private var m_Dir:int;				    // The direction to move once m_MoveCountdown hits zero.
	
		private var m_GameState:int;			// Are we splash screen, game mode, or game over.
		private var m_Paused:Boolean;
		
		private const SPLASH:int = 0;
		private const GAME:int = 1;
		private const GAMEOVER:int = 2;
		
		private const UP:int = 38;				// Classic code readability trick. m_Dir will be set to one of these.
		private const DOWN:int = 40;
		private const LEFT:int = 37;
		private const RIGHT:int = 39;
		
		private const SPEED:int = 4;			// How many frames for each move.
		private const GRID_SIZE:int = 16;
		private const PIX_PER_GRID:int = 16;

		public function Main() // Called when game starts runing.
		{
			m_Paused = true;
			
			// These functions will be called each time the event happens.
			stage.addEventListener(Event.ENTER_FRAME,      Update);
   			stage.addEventListener(KeyboardEvent.KEY_DOWN, OnKeyDown);
			stage.addEventListener(Event.DEACTIVATE, 	   WindowNotActive);
		    stage.addEventListener(Event.ACTIVATE, 		   WindowActive);
			
			m_StatusText = new TextField();
			m_StatusText.width = 256;
			m_StatusText.y = 128-64;
			m_StatusText.selectable = false;
			
			var format:TextFormat = new TextFormat();
            format.font = "Courier";
            format.color = 0xc0c0c0;
            format.size = 24;
            format.align = TextFormatAlign.CENTER;
			
			m_StatusText.defaultTextFormat = format;
            m_StatusText.setTextFormat(format);

			
			m_GameState = SPLASH; // Start in splash screen mode.			
			m_MoveCountdown = SPEED; // Start the countdown.
			m_Dir = UP; // Start moving up.
			
			// Sigh, the following disaster is how to make a multidimensional array in as3. Please let me know if there is a non-horrible way to do this.
			m_Grid = new Array(GRID_SIZE);
			for( var i = 0; i < GRID_SIZE; i++ )
			{
				m_Grid[i] = new Array(GRID_SIZE);
				
				for( var j = 0; j < GRID_SIZE; j++ )
				{
					m_Grid[i][j] = null; // Init the whole array.
				}
			}
			
			m_SnakeList = new Array();
			
			NewGame();
			
			stage.addChild(m_StatusText);
			SetTextForStatus();
		}
		
		public function NewGame():void
		{
			m_Fruit = null;
			m_SnakeList.splice(0); // This clears the array (or so I've read)
			
			for( var i = 0; i < GRID_SIZE; i++ )
			{
				for( var j = 0; j < GRID_SIZE; j++ )
				{
					if(m_Grid[i][j])
					{
						stage.removeChild(m_Grid[i][j]);
					}
					m_Grid[i][j] = null;
				}
			}
			
			var newHead:Segment = new Segment();
			stage.addChild(newHead);
			m_SnakeList.push(newHead);
			MoveToGrid(GRID_SIZE/2,GRID_SIZE/2,newHead);
			
			m_Fruit = new Fruit();
			stage.addChild(m_Fruit);
			
			MoveToGrid(0,0,m_Fruit); // This means the fruit can't start at 0,0. But no one will care.
			PlaceFruit();
		}
		
		public function Update(event:Event):void
		{
			if(m_GameState == SPLASH)
			{
				
			}
			else if(m_GameState == GAME && !m_Paused)
			{
				m_MoveCountdown--; // Take one away from the countdown.
				if(m_MoveCountdown <= 0) // Countdown is over. 
				{
					var snakeX:int = GetGridXIndex(m_SnakeList[0]);
					var snakeY:int = GetGridYIndex(m_SnakeList[0]);
					
					if(m_Dir == UP)
					{
						snakeY--;
						
						if(snakeY < 0)
						{
							snakeY += GRID_SIZE;
						}
					}
					else if(m_Dir == DOWN)
					{
						snakeY++;
						
						if(snakeY >= GRID_SIZE)
						{
							snakeY -= GRID_SIZE;
						}
					}
					else if(m_Dir == LEFT)
					{
						snakeX--;
						
						if(snakeX < 0)
						{
							snakeX += GRID_SIZE;
						}
					}
					else if(m_Dir == RIGHT)
					{
						snakeX++;
						
						if(snakeX >= GRID_SIZE)
						{
							snakeX -= GRID_SIZE;
						}
					}
					
					var removeTail:Boolean = true;
					
					if(m_Grid[snakeX][snakeY] != null)
					{
						// I'll concede that this type checking syntax is lovely... its probably madly inneficent though.
						// I would have had the IDE syntax hilight "is" to blue. IDK why they don't do that.
						if(m_Grid[snakeX][snakeY] is Fruit)
						{
							PlaceFruit();
							removeTail = false; // This will grow the snake by 1
						}
						else if(m_Grid[snakeX][snakeY] is Segment)
						{
							m_GameState = GAMEOVER// Game Over
							SetTextForStatus();
						}
						
					}
					
					if(m_GameState != GAMEOVER)
					{
						var newHead:Segment = new Segment();	// Create a new segment to put at the front.
						stage.addChild(newHead);
						
						MoveToGrid(snakeX,snakeY,newHead); // Put the new segment at the right spot.
						
						m_SnakeList.unshift(newHead); // Put the new segment at the front of the list.
					
						if(removeTail)
						{
							var SnakeTail:Segment = m_SnakeList.pop(); // Get the end of the list and remove it.
							RemoveFromGrid(SnakeTail);
							stage.removeChild(SnakeTail);
							// In a real language you would have to free the memory associated with SnakeTail... 
							// I'm under the understanding that as3 has garbage collection and does it for you.
							// Otherwise this is a memory leak.
						}
					}
					
					
					m_MoveCountdown = SPEED; // Restart the countdown.
				}
				else if(m_GameState == GAMEOVER)
				{
					
				}
			}
		}
		
		public function MoveToGrid(gridX:int, gridY:int, toMove:MovieClip):void
		{
			toMove.x = gridX * PIX_PER_GRID + PIX_PER_GRID/2;
			toMove.y = gridY * PIX_PER_GRID + PIX_PER_GRID/2;
			m_Grid[gridX][gridY] = toMove;
		}
		
		public function RemoveFromGrid(toRemove:MovieClip):void
		{
			var gridX:int = GetGridXIndex(toRemove);
			var gridY:int = GetGridYIndex(toRemove);
			m_Grid[gridX][gridY] = null;
		}
		
		public function GetGridXIndex(test:MovieClip):int
		{
			return (test.x - PIX_PER_GRID/2)/PIX_PER_GRID;
		}
		
		public function GetGridYIndex(test:MovieClip):int
		{
			return (test.y - PIX_PER_GRID/2)/PIX_PER_GRID;
		}
		
		public function PlaceFruit():void
		{	
			// This random function is wonky to me. Not sure if I'm doin it right.
			var fruitX:int = Math.round(Math.random()*(GRID_SIZE-1));
			var fruitY:int = Math.round(Math.random()*(GRID_SIZE-1));
			
			while(m_Grid[fruitX][fruitY] != null) // Look for an empty spot... I suppose this might hang if the snake covers the whole playing field.
			{
				fruitX = Math.round(Math.random()*(GRID_SIZE-1));
				fruitY = Math.round(Math.random()*(GRID_SIZE-1));
			}
			
			RemoveFromGrid(m_Fruit);
				
			MoveToGrid(fruitX,fruitY,m_Fruit);
		}
		
		public function SetTextForStatus():void
		{
			if(m_Paused)
			{
				m_StatusText.text = "Paused\nClick the window\nto resume."
			}
			else if(m_GameState == SPLASH)
			{
				m_StatusText.text = "Snake\nWASD To Move";
			}
			else if(m_GameState == GAMEOVER)
			{
				m_StatusText.text = "Game Over\nPress space for new\ngame."
			}
			else
			{
				m_StatusText.text = "";
			}
			stage.setChildIndex(m_StatusText,stage.numChildren - 1);
		}
		
		private function OnKeyDown(e:KeyboardEvent):void
   	    {	
			if(m_Paused)
			{
				return; // This should be impossible.
			}
			
			if(m_GameState == GAMEOVER)
			{
				if(e.keyCode == 32) // Space key.
				{
					m_GameState = GAME;
					NewGame();
					SetTextForStatus();
				}
				
				return;
			}
			
			var Moved:Boolean = false;
			
			if(e.keyCode == UP || e.keyCode == 87) // Up (Up arrow or 'w')
			{
				if(m_Dir != DOWN || m_SnakeList.length <= 1) // Don't let the player suicide by reversing direction.
				{
					m_Dir = UP;
				}
				Moved = true;
			} 
			else if(e.keyCode == DOWN || e.keyCode == 83) // Down (Down arrow or 's')
			{
				if(m_Dir != UP || m_SnakeList.length <= 1) // Don't let the player suicide by reversing direction.
				{
					m_Dir = DOWN;
				}
				Moved = true;
			} 
			else if(e.keyCode == LEFT || e.keyCode == 65) // Left (Left arrow or 'a') 
			{
				if(m_Dir != RIGHT || m_SnakeList.length <= 1) // Don't let the player suicide by reversing direction.
				{
					m_Dir = LEFT;
				}
				Moved = true;
			} 
			else if(e.keyCode == RIGHT || e.keyCode == 68) // Right (Right arrow or 'd')  
			{
				if(m_Dir != LEFT || m_SnakeList.length <= 1) // Don't let the player suicide by reversing direction.
				{
					m_Dir = RIGHT;
				}
				Moved = true;
			}
			
			if(Moved && m_GameState == SPLASH)
			{
				m_GameState = GAME;
				SetTextForStatus();
			}
        }
		
		public function WindowNotActive(event:Event):void
		{
			m_Paused = true;
			
			SetTextForStatus();
		}
		
		public function WindowActive(event:Event):void
		{
			m_Paused = false;
			
			SetTextForStatus();
		}
	}
}